package com.sun.org.apache.xml.internal.utils;

import com.sun.org.apache.xml.internal.res.XMLMessages;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.TransformerException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import java.io.PrintStream;
import java.io.PrintWriter;
import javax.xml.transform.ErrorListener;
import org.xml.sax.ErrorHandler;

public class DefaultErrorHandler implements ErrorHandler, ErrorListener
{
    PrintWriter m_pw;
    boolean m_throwExceptionOnError;
    
    public DefaultErrorHandler(PrintWriter pw) {
        this.m_throwExceptionOnError = true;
        this.m_pw = pw;
    }
    
    public DefaultErrorHandler(PrintStream pw) {
        this.m_throwExceptionOnError = true;
        this.m_pw = new PrintWriter(pw, true);
    }
    
    public DefaultErrorHandler() {
        this(true);
    }
    
    public DefaultErrorHandler(boolean throwExceptionOnError) {
        this.m_throwExceptionOnError = true;
        this.m_pw = new PrintWriter(System.err, true);
        this.m_throwExceptionOnError = throwExceptionOnError;
    }
    
    @Override
    public void warning(SAXParseException exception) throws SAXException {
        printLocation(this.m_pw, exception);
        this.m_pw.println("Parser warning: " + exception.getMessage());
    }
    
    @Override
    public void error(SAXParseException exception) throws SAXException {
        throw exception;
    }
    
    @Override
    public void fatalError(SAXParseException exception) throws SAXException {
        throw exception;
    }
    
    @Override
    public void warning(TransformerException exception) throws TransformerException {
        printLocation(this.m_pw, exception);
        this.m_pw.println(exception.getMessage());
    }
    
    @Override
    public void error(TransformerException exception) throws TransformerException {
        if (this.m_throwExceptionOnError) {
            throw exception;
        }
        printLocation(this.m_pw, exception);
        this.m_pw.println(exception.getMessage());
        return;
    }
    
    @Override
    public void fatalError(TransformerException exception) throws TransformerException {
        if (this.m_throwExceptionOnError) {
            throw exception;
        }
        printLocation(this.m_pw, exception);
        this.m_pw.println(exception.getMessage());
        return;
    }
    
    public static void ensureLocationSet(TransformerException exception) {
        SourceLocator locator = null;
        Throwable cause = exception;
        do {
            if ((cause instanceof SAXParseException)) {
                locator = new SAXSourceLocator((SAXParseException)cause);
            }
            else if ((cause instanceof TransformerException)) {
                SourceLocator causeLocator = ((TransformerException)cause).getLocator();
                if (null != causeLocator) {
                    locator = causeLocator;
                }
            }
            if ((cause instanceof TransformerException)) {
                cause = ((TransformerException)cause).getCause();
            }
            else if ((cause instanceof SAXException)) {
                cause = ((SAXException)cause).getException();
            }
            else {
                cause = null;
            }
        } while (null != cause);
        exception.setLocator(locator);
    }
    
    public static void printLocation(PrintStream pw, TransformerException exception) {
        printLocation(new PrintWriter(pw), exception);
    }
    
    public static void printLocation(PrintStream pw, SAXParseException exception) {
        printLocation(new PrintWriter(pw), exception);
    }
    
    public static void printLocation(PrintWriter pw, Throwable exception) {
        SourceLocator locator = null;
        Throwable cause = exception;
        do {
            if ((cause instanceof SAXParseException)) {
                locator = new SAXSourceLocator((SAXParseException)cause);
            }
            else if ((cause instanceof TransformerException)) {
                SourceLocator causeLocator = ((TransformerException)cause).getLocator();
                if (null != causeLocator) {
                    locator = causeLocator;
                }
            }
            if ((cause instanceof TransformerException)) {
                cause = ((TransformerException)cause).getCause();
            }
            else if ((cause instanceof WrappedRuntimeException)) {
                cause = ((WrappedRuntimeException)cause).getException();
            }
            else if ((cause instanceof SAXException)) {
                cause = ((SAXException)cause).getException();
            }
            else {
                cause = null;
            }
        } while (null != cause);
        if (null != locator) {
            String id = (null != locator.getPublicId()) ? locator.getPublicId() : ((null != locator.getSystemId()) ? locator.getSystemId() : XMLMessages.createXMLMessage("ER_SYSTEMID_UNKNOWN", (Object[])null));
            pw.print(id + "; " + XMLMessages.createXMLMessage("line", (Object[])null) + locator.getLineNumber() + "; " + XMLMessages.createXMLMessage("column", (Object[])null) + locator.getColumnNumber() + "; ");
        }
        else {
            pw.print("(" + XMLMessages.createXMLMessage("ER_LOCATION_UNKNOWN", (Object[])null) + ")");
        }
    }
}
